
#include <windows.h>
#include <iostream>
#include "cv.h"
#include "cvaux.h"
#include "highgui.h"

using namespace cv;
using namespace std;

#include "BlobResult.h"

#include "common.h"
#include "auto_eye_tracker.h"
#include "queue.h"

#include "eye_window_detector.h"


static void show_blobs(CBlobResult* ps_blobs, char* window_name, int rows, int cols);
static CBlobResult* merge_close_by_regions(CBlobResult* ps_blobs, int rows, int cols, int w, int h);

//Thread function. Strictly a dummy placeholder!
static DWORD WINAPI thread(LPVOID pv_params)
{
	eye_window_detector	*ctxt = (eye_window_detector*)pv_params;

	ctxt->process();

	return 0;
}


eye_window_detector::eye_window_detector(  int eye_window_h, 
										   int eye_window_w, 
										   queue_t* in_original,
										   queue_t* in_segmented,
										   queue_t* in_scene,
										   queue_t* out_original,
										   queue_t* out_processed,
										   queue_t* out_scene)
										   
{
	this->h = eye_window_h;
	this->w = eye_window_w;

	this->in_original = in_original;
	this->in_segmented = in_segmented;
	this->out_original = out_original;
	this->out_coords = out_processed;

	this->in_scene = in_scene;
	this->out_scene = out_scene;

	this->last_coords = Point(-1,-1);

	this->keep_running = true;

		//Create the thread.
	this->h_thread = CreateThread(	NULL,                   // default security attributes
									0,                      // use default stack size  
									thread,					// thread function name
									this,					// argument to thread function 
									0,                      // use default creation flags 
									&this->thread_id);		// returns the thread identifier 

	if(NULL == this->h_thread)
	{
		cout << "Error in creating Eye-windows thread\n";
		return;
	}

	cout << "EWD created!\n";
}


eye_window_detector::~eye_window_detector(void)
{
	//Signal the thread to close.
	this->keep_running = false;
	
	// Wait for it to finish.
	WaitForSingleObject(this->h_thread, INFINITE);

	//That is it!
	cout << "Eye-window detector closed.\n";

}

//Actual thread function.
status_t eye_window_detector::process(void)
{
	status_t	error;

	Mat *ps_captured_frame = NULL;
	Mat *ps_seg = NULL;
	Mat *ps_scene;

	CBlobResult		s_blobs;
	CBlobResult*	ps_filtered_blobs;
	IplImage		s_image;
	Point			*ps_eye_coords; //Approximate eye location.

	int newh, neww;
	int size_limit;

	unsigned int start;

	while( this->keep_running)
	{
		//Grab input frames.
		ps_captured_frame = NULL;
		ps_seg = NULL;
		
		ps_eye_coords = new Point(-1, -1);
		
		GETBUF(in_original, ps_captured_frame, 100);
		/*do\
		{\
			error = this->in_original->dequeue((void**)&ps_captured_frame, 100);\
		}while(TIMEOUT == error && this->keep_running);*/

		if(SUCCESS != error || ps_captured_frame == NULL) 
		{
			cout << "EWD: Error in reading input frame, " << error <<endl;
			continue;
		}

		if( ps_captured_frame->empty() )
			continue;

		GETBUF(in_segmented, ps_seg, 100);

		if(SUCCESS != error || NULL == ps_seg) 
		{
			cout << "Error in reading segmented frame\n";
			delete ps_captured_frame;
			continue;
		}

		if( ps_seg->empty() )
		{
			delete ps_captured_frame;
			continue;
		}

		start = clock();

		//We have both inputs now.
		//Re-calculate the h and w params for the resized image.
		newh = (h * ps_seg->rows)/ps_captured_frame->rows;
		neww = (w * ps_seg->cols)/ps_captured_frame->cols;

		//Step 0. CV blobs wants IplImages, not Mat.
		s_image = *ps_seg;

		/* Step 1. Perform connected-component labelling, using CVblobs
		   s_blobs contains a list of all the components in the image. */
		
		s_blobs = CBlobResult(&s_image, NULL, 0);

		// Discard objects that are too big, width-wise.
		s_blobs.Filter(s_blobs, B_INCLUDE, CBlobGetDiffX(), B_LESS, MULT_FACTOR*neww);

		// Discard objects that are too big, height-wise.
		s_blobs.Filter(s_blobs, B_INCLUDE, CBlobGetDiffY(), B_LESS, MULT_FACTOR*newh);

		// Discard objects that are too small, width-wise.
		s_blobs.Filter(s_blobs, B_EXCLUDE, CBlobGetDiffX(), B_LESS, neww/4);
		
		// Discard objects that are too small, height-wise.
		s_blobs.Filter(s_blobs, B_EXCLUDE, CBlobGetDiffY(), B_LESS, newh/4);

		
		// Discard objects that are do not have the correct aspect ratio (2:1) or thereabouts.
		// Calculated by looking at the orientation of the object.
		s_blobs.Filter(s_blobs, B_EXCLUDE, CBlobGetOrientation(), B_GREATER, ANGLE_THRESH);

		/*{
			int idx = 0;
			cout << "After filter\n";
			cout << "Num blobs: "<< s_blobs.GetNumBlobs() <<endl;
			for(idx =0 ;idx < s_blobs.GetNumBlobs(); idx++)
			{
				CBlob blob = s_blobs.GetBlob(idx);
				printf("Blob: %d: %d by %d at (%d, %d)\n",
					   idx, blob.GetBoundingBox().width, blob.GetBoundingBox().height,
					   blob.GetBoundingBox().x, blob.GetBoundingBox().y);
			}
		}*/

		
		//If there are multiple blobs left
		//Take out the top blob, if the top one is too high - it is probably an eyebrow.
		if( s_blobs.GetNumBlobs() > 1)
		{
			int j;

			CBlob top_most, second;
			s_blobs.GetNthBlob( CBlobGetMaxY(), s_blobs.GetNumBlobs()-1, top_most );
			s_blobs.GetNthBlob( CBlobGetMaxY(), s_blobs.GetNumBlobs()-2, second );

			//cout << "TOP: " << top_most.MaxY() <<" second: " << second.MaxY()  << endl;

			//As Y grows downwards, second.y > top_most.y
			if( second.MaxY()  - top_most.MaxY() > (newh/4))
			{
				//delete this blob from the list - how?
				s_blobs.Filter(s_blobs, B_EXCLUDE, CBlobGetMaxY(), B_LESS, top_most.MaxY()+1);
			}

			if( s_blobs.GetNumBlobs() > 1)
			{
				//If there are still lots of blobs, select the one with the largest area.
				s_blobs.GetNthBlob( CBlobGetArea(), 0, top_most );
		
				//Should give us only one blob!
				s_blobs.Filter(s_blobs, B_EXCLUDE, CBlobGetArea(), B_LESS, top_most.Area());
			}
		}

		//show_blobs(&s_blobs, "Eye_window_Filtered", ps_seg->rows, ps_seg->cols);

		//At this point, we should have exactly 1 blob left. If everything has gone
		//well, this will be the eye region.

		//Find the x and y means.
		if( s_blobs.GetNumBlobs() > 0)
		{
			CBlob eye_blob;
			int eye_x;
			int eye_y;
			
			s_blobs.GetNthBlob( CBlobGetArea(), 0, eye_blob );

			eye_x = eye_blob.Moment(1,0) / eye_blob.Moment(0,0);
			eye_y = eye_blob.Moment(0,1) / eye_blob.Moment(0,0);

			//Extrapolate to the actual image.
			eye_x = (eye_x * ps_captured_frame->cols)/(ps_seg->cols);
			eye_y = (eye_y * ps_captured_frame->rows)/(ps_seg->rows);

			//Smoothen it, so that random jumps do not occur.
			if( this->last_coords.x == -1)
			{
				ps_eye_coords->x = eye_x;
				ps_eye_coords->y = eye_y;
			}
			else
			{
				ps_eye_coords->x = ALPHA * this->last_coords.x + (1-ALPHA)*eye_x;
				ps_eye_coords->y = ALPHA * this->last_coords.y + (1-ALPHA)*eye_y;

			}

			//store for next time
			this->last_coords.x = ps_eye_coords->x;
			this->last_coords.y = ps_eye_coords->y;
		
			////Draw a circle at the eye, in white
			//circle(*ps_captured_frame, *ps_eye_coords, 5, Scalar(255,255,255), -1);

			//imshow("eye_pos", *ps_captured_frame);
			//waitKey(1);
		}
		else
		{
			//Apparently, there are no eye regions left.
			//So, do not push out anything.
			delete ps_seg;
			delete ps_captured_frame;
			continue;
		}

		//Delete the segmented input frame; nobody down the line uses it.
		delete ps_seg;
		ps_seg = NULL;

		//printf("Time taken: %d\n", clock() - start);

		//push the original frame, and the eye coordinates.
		if( this->out_original)
		{
			error = this->out_original->enqueue(ps_captured_frame, 100);
			if( SUCCESS != error)
			{
				if( TIMEOUT != error)
					cout << "EWD: Unable to push out input frame " << error << endl;
				delete ps_captured_frame;
				ps_captured_frame = NULL;
				continue;
			}
		}
		else
		{
			delete ps_captured_frame;
			ps_captured_frame = NULL;
		}

		//Push out the coordinates.
		if( this->out_coords)
		{
			error = this->out_coords->enqueue(ps_eye_coords, 100);
			if( SUCCESS != error)
			{
				if( TIMEOUT != error)
					cout << "EWD: Unable to push out coordinates " << error << endl;
				delete ps_eye_coords;
				ps_eye_coords = NULL;
				continue;
			}
		}
		else
		{
			delete ps_eye_coords;
			ps_eye_coords = NULL;
		}


		//Pull and push a scene frame.
		if( this->in_scene)
		{
			GETBUF(in_scene, ps_scene, 100);

			if(SUCCESS != error || NULL == ps_scene) 
			{
				cout << "EWD:Error in reading scene frame\n";
				delete ps_scene;
				continue;
			}

			if( this->out_scene)
			{
				error = this->out_scene->enqueue(ps_scene, 100);
				if( SUCCESS != error)
				{
					if( TIMEOUT != error)
						cout << "EWD: Unable to push out scene " << error << endl;
					delete ps_scene;
					continue;
				}
			}
			else
			{
				delete ps_scene;
				continue;
			}
		}
	}

	return SUCCESS;
}


//A small function to display blobs.
static void show_blobs(CBlobResult* ps_blobs, char* window_name, int rows, int cols)
{
	int i;
	CBlob *temp_blob;
			
	Mat temp = Mat::zeros(rows, cols, CV_8UC1);
	IplImage temp_image = temp;

	for (i = 0; i < ps_blobs->GetNumBlobs(); i++ )
	{
		temp_blob = ps_blobs->GetBlob(i);
		temp_blob->FillBlob( &temp_image, CV_RGB(255,255,255));
	}

	imshow(window_name, Mat(&temp_image));
	waitKey(1);
}	


//This function is not used, for now. Instead, the image is smeared
//using a Gaussian blur filter.

static CBlobResult* merge_close_by_regions(CBlobResult* ps_blobs, int rows, int cols, int w, int h)
{
	//Re-create the image.
	int i, x, y;
	CBlob *temp_blob;

	int xlow  = w+1;
	int xhigh = cols - w-1;
	int ylow  = h+1;
	int yhigh = rows - h-1;

	int sum;
	int den = (w * h) / 4;
			
	Mat s_seg = Mat::zeros(rows, cols, CV_8UC1);
	IplImage temp = s_seg;

	for (i = 0; i < ps_blobs->GetNumBlobs(); i++ )
	{
		temp_blob = ps_blobs->GetBlob(i);
		temp_blob->FillBlob( &temp, CV_RGB(1,1,1));
	}

	s_seg = Mat(&temp);
	Mat *ps_seg = &s_seg;

	//Go through the pixels. If there are enough pixels around it that are high,
	//Mark this also as a good pixel.
	for (y = ylow; y < yhigh; y++)
	{
		for(x = xlow; x < xhigh; x++)
		{
			 sum = find_sum(ps_seg, x-w/4, 	y-h/4, h/2, w/2);

			 if( sum / den > 0.35 )
			 {
				 PIXEL(ps_seg, x, y) = 1;
			 }
		}
	}

	temp = s_seg;

	return new CBlobResult(&temp, NULL, 0);
}

