/********************************************************************
find_eye_analogues.c Compute possible Eye-Windows from a face image.
v1.0 - Initial version.											
********************************************************************/

#include "cv.h"
#include "cvaux.h"
#include "highgui.h"

using namespace cv;

#include "auto_eye_tracker.h"

#define MEAN_HITS_THRESHOLD		6

void find_eye_analogue(	Mat *ps_input,
						Mat	*ps_eye_analogue,
						int 		h, 
						int			w);

static unsigned char find_mean(	Mat *ps_input, 
			  		int x_pos, 
					int y_pos, 
					int	h, 
					int	w);
						
static unsigned int sum_rectangle( Mat *ps_input, 
			  				int x_pos, 
							int y_pos, 
							int	h, 
							int	w);

static void clear_boundaries(Mat *ps_eye_analogue, 
					  int left, 
					  int right, 
					  int top, 
					  int bottom);						
						

/**************************************************************************************
find_eye_analogue: Calculates possible eye-analogue pixels using mean values.

Inputs: ps_input: Input frame with an eye image.
		ps_eye_analogue: Output frame. It is assumed that this frame has been
						 allocated; it is overwritten.
		h : Height of eye window. Should be an even number of pixels.
		w : Width of eye window. Should be an even number of pixels.
**************************************************************************************/
void find_eye_analogue_dumb(	Mat *ps_input, 
						Mat	*ps_eye_analogue,
						int  h, 
						int	 w)
{
	int x, y;
	int num_hits;
	int cur_pixel;

	int half_w = w/2;
	int half_h = h/2;

	int nrows = ps_input->rows;
	int ncols = ps_input->cols;

	int xlow, xhigh, ylow, yhigh;


	//Clear the output image to all zeros.
	ps_eye_analogue->setTo(Scalar(0), 
						   Mat::ones(nrows, ncols, CV_8UC1) );

	xlow  = half_w+1;
	xhigh = ncols - half_w-1;
	ylow  = half_h+1;
	yhigh = nrows - half_h-1;


//	for( y = ylow; y < yhigh; y++)
	for( y = yhigh-1; y >= ylow; y--)
	{
		//for(x = xlow ; x < xhigh; x++)
		for( x = xhigh-1; x >= xlow; x--)
		{
			int num_test = 2;
			num_hits = 0;
			cur_pixel = PIXEL(ps_input, x, y); //ps_input->u[y][x];
			
		//	DBG "Eye-analogue:Test %d: Searching around (%d, %d) (%d) ...\n", num_test, x, y, cur_pixel);
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x-half_w, y+half_h, half_h, half_w)) 		//2
				num_hits++ ;
			
			num_test++;
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x,   y+half_h, half_h, 1)) 		//3
				num_hits++ ;
			
			num_test++;
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x+1, y+half_h, half_h, half_w)) 		//4
				num_hits++ ;
			num_test++;
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x-half_w, y, 1, half_w)) 		//5
				num_hits++ ;
			num_test++;
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x+1, y, 1, half_w)) 		//6
				num_hits++ ;
			num_test++;
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x-half_w, y-1, half_h, half_w)) 		//7
				num_hits++ ;
			num_test++;
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x, y-1, half_h, 1)) 		//8
				num_hits++ ;
			num_test++;	
			
			if ( cur_pixel < 0.9 * find_mean(ps_input, x+1, y-1, half_h, half_w)) 		//9
				num_hits++ ;
			num_test++;				
			

			PIXEL(ps_eye_analogue, x, y) = (num_hits >= MEAN_HITS_THRESHOLD)? 255: 0;
			
		//	DBG "Eye-analogue: Count (%d, %d, %d) = %d; output pixel = %d\n",\
				x, y, cur_pixel, num_hits,\
				*(ps_eye_analogue->data + ps_eye_analogue->step.buf[0]*y + ps_eye_analogue->step.buf[1]*x));
		}
	}
}

//Calculate mean around a pixel.

static unsigned char find_mean( Mat *ps_input, 
			  			int x_pos, 
						int y_pos, 
						int	h, 
						int	w)
{
	//unsigned char mean1 = (int)(sum_rectangle(ps_input, x_pos, y_pos, h, w))/ (h+w);
	
	//DBG "Mean of ( %d, %d ), hw = %d, %d = %d\n", x_pos, y_pos, h, w, mean1);

	if( x_pos < 0 || y_pos < h || y_pos < 0)
	{
		printf("trouble, x is less than 0\n");
	}

	Mat rows = ps_input->rowRange(y_pos-h, y_pos);
	Mat small = rows.colRange(x_pos, x_pos+w);
	Scalar mean2 = mean(small);
	unsigned char mean1 = mean2.val[0];
	
	return mean1;
}

//Calculate sum around a pixel.
static unsigned int sum_rectangle(Mat *ps_input, 
			  			   int x_pos, 
						   int y_pos, 
						   int	h, 
						   int	w)
{
	int x,y;
	int sum = 0;


	// Scroll through the image, computing sum.
	for( y = y_pos; y > y_pos - h; y--)
	{
		for(x = x_pos; x < x_pos + w ; x++)
		{
			sum += PIXEL(ps_input, x, y);//ps_input->u[y][x];
		}
	}
		
	//DBG "Sum at (%d,%d) = %d\n", x,y, sum);
	
	return sum;
}

#if 0
void clear_boundaries(VisXimage_t *ps_eye_analogue, 
					  int left, 
					  int right, 
					  int top, 
					  int bottom)
{
	
	int x, y;
	
	int xmin = ps_eye_analogue->xlo + left; 
	int xmax = ps_eye_analogue->xhi - right;
	int ymin = ps_eye_analogue->ylo + bottom; 
	int ymax = ps_eye_analogue->yhi - top;
	
	//Loop 1: Left strip.
	for(y = ps_eye_analogue->ylo; y < ps_eye_analogue->yhi; y++)
	{
		for(x = ps_eye_analogue->xlo; x < xmin; x++)
		{
			ps_eye_analogue->u[y][x] = 0;
		}
	}
	
	//Right strip
	for(y = ps_eye_analogue->ylo; y < ps_eye_analogue->yhi; y++)
	{
		for(x = xmax; x < ps_eye_analogue->xhi; x++)
		{
			ps_eye_analogue->u[y][x] = 0;
		}
	}
	
	//Bottom
	for(y = ps_eye_analogue->ylo; y < ymin; y++)
	{
		for(x = ps_eye_analogue->xlo; x < ps_eye_analogue->xhi; x++)
		{
			ps_eye_analogue->u[y][x] = 0;
		}
	}
	
	//Top
	for(y = ymax; y < ps_eye_analogue->yhi; y++)
	{
		for(x = ps_eye_analogue->xlo; x < ps_eye_analogue->xhi; x++)
		{
			ps_eye_analogue->u[y][x] = 0;
		}
	}

	
}
#endif
