// auto_eye_tracker.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
using namespace std;

#include "cv.h"
#include "cvaux.h"
#include "highgui.h"

using namespace cv;
#include "common.h"
#include "auto_eye_tracker.h"

#include "queue.h"
#include "eye_analogue.h"
#include "video_capture.h"
#include "eye_window_detector.h"
#include "hough_transform.h"
#include "compositor.h"

int _tmain(int argc, _TCHAR* argv[])
{
	//Create input and output queues.
	queue_t		*q_vcap_eyean_eye = new queue_t;
	queue_t		*q_cap2_eyean_scene =  new queue_t;
	queue_t		*q_eyean_ewd_eye = new queue_t;
	queue_t		*q_eyean_ewd_scene = new queue_t;
	queue_t		*q_eyean_ewd_blobs = new queue_t;
	queue_t		*q_ewd_hough_eye = new queue_t;
	queue_t		*q_ewd_hough_scene = new queue_t;
	queue_t		*q_ewd_hough_coords = new queue_t;
	queue_t		*q_hough_comp_coords = new queue_t;
	queue_t		*q_hough_comp_scene = new queue_t;
	
	video_capture *ps_vcap[2];
	int eye_camera;
	int scene_camera;

	printf("------------------------------------------------------\n");
	printf("                 Eye Tracker\n");
	printf("          Anil Ram Viswanathan / Zelan Xiao\n");
	printf("             Cornell University 2011\n");
	printf("------------------------------------------------------\n\n");

	printf("Starting cameras...\n");

	//Create the vcap nodes.
	ps_vcap[0] = new video_capture(1, DEFAULT_W, DEFAULT_H);
	ps_vcap[1] = new video_capture(2, DEFAULT_W, DEFAULT_H);

	//Start the capture.
	ps_vcap[0]->start();
	ps_vcap[1]->start();

	//Select the eye camera.
	printf("Select the eye camera (1 or 2)\n");
	scanf("%d", &eye_camera);
	eye_camera--; //Bring it back to 1 or zero (idx values, instead of camera values).
	scene_camera = 1 - eye_camera;
	
	//Set the output queue for the video capture nodes.
	ps_vcap[eye_camera]->set_output_queue(q_vcap_eyean_eye);
	ps_vcap[scene_camera]->set_output_queue(q_cap2_eyean_scene);

	//Create the eye-analogue finder.
	eye_analogue *ps_ea = new eye_analogue(DEFAULT_H, DEFAULT_W, 
										   q_vcap_eyean_eye, q_cap2_eyean_scene,
										   q_eyean_ewd_scene,
										   q_eyean_ewd_eye,
										   q_eyean_ewd_blobs);
	//Create the eye-window detector.
	eye_window_detector *ps_ewd = new eye_window_detector(DEFAULT_H, DEFAULT_W, 
										   q_eyean_ewd_eye,
										   q_eyean_ewd_blobs,
										   q_eyean_ewd_scene,
										   q_ewd_hough_eye,
										   q_ewd_hough_coords,
										   q_ewd_hough_scene);
														  

	//Create the hough-transform node.
	hough_transform *ps_hough = new hough_transform(DEFAULT_H, DEFAULT_W,
										   q_ewd_hough_eye,
										   q_ewd_hough_coords,
										   q_ewd_hough_scene,
										   q_hough_comp_coords,
										   q_hough_comp_scene);
										   
	//Create the bright-spot detector.
	compositor	*ps_comp = new compositor("session1", q_hough_comp_scene, q_hough_comp_coords);

	ps_comp->set_mode(COMP_CALIBRATE);
	

	char choice = 'a';

	while( 'q' != choice)
	{
		//Main menu.
		cout << "------------------------------------------------\n";
		cout << "       Eye Tracker Main menu\n";
		cout << "c. Switch to CALIBRATE mode\n";
		cout << "n. Calibrate NOW\n";
		cout << "t. Switch to TRACK mode\n";
		cout << "q. Quit\n";
		cout << "Enter choice: ";
		cin >> choice;

		switch(choice)
		{
			case 'c':
			case 'C':
			{
				cout <<"Switching to CALIBRATE mode\n";
				ps_comp->set_mode(COMP_CALIBRATE);
				break;
			}
			case 'n':
			case 'N':
			{
				cout <<"Calibrating NOW\n";
				ps_comp->calibrate_on_next_frame();
				break;
			}
			case 't':
			case 'T':
			{
				printf("Switching to tracking mode\n");
				ps_comp->set_mode(COMP_TRACK);
				break;
			}
			case 'q':
			case 'Q':
			{
				cout << "Hasta la vista!\n";
				break;
			}
			default:
			{
				cout << "Invalid choice; try again\n";
			}
		}
	}	


	delete ps_comp;
	delete ps_hough;
	delete ps_ewd;
	delete ps_ea;
	delete ps_vcap[1];
	delete ps_vcap[0];

	delete q_vcap_eyean_eye;
	delete q_cap2_eyean_scene;
	delete q_eyean_ewd_eye;
	delete q_eyean_ewd_scene;
	delete q_eyean_ewd_blobs;
	delete q_ewd_hough_eye;
	delete q_ewd_hough_scene;
	delete q_ewd_hough_coords;
	delete q_hough_comp_coords;
	delete q_hough_comp_scene;

	
  	exit(0);
}
